/*
	Plussy Display
	Copyright (C) 2015  Christian Carlowitz <chca@cmesh.de>

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "common.h"

void h2rgb(uint8_t* rgb, uint8_t h) // h = HSV with hue only ;-)
{
	uint8_t i = h/43; // roughly /(255/6)
	uint8_t f = h%43;
	uint8_t p = 0;
	uint8_t q = 255-f*6;
	uint8_t t = f*6;
	switch(i)
	{
		case 0: rgb[0] = 255; rgb[1] = t; rgb[2] = p; break;
		case 1: rgb[0] = q; rgb[1] = 255; rgb[2] = p; break;
		case 2: rgb[0] = p; rgb[1] = 255; rgb[2] = t; break;
		case 3: rgb[0] = p; rgb[1] = q; rgb[2] = 255; break;
		case 4: rgb[0] = t; rgb[1] = p; rgb[2] = 255; break;
		default: rgb[0] = 255; rgb[1] = p; rgb[2] = q; break;
	}
}

void hsv2rgb_f(uint8_t* rgb, float h, float s, float v)
{
	// https://de.wikipedia.org/wiki/HSV-Farbraum#Umrechnung_HSV_in_RGB
	unsigned int hi = (unsigned int)(h/60.0f);
	float f = h/60.0f - hi;
	float p = v*(1.0f - s);
	float q = v*(1.0f - s*f);
	float t = v*(1.0f - s*(1.0f-f));
	
	float r=0,g=0,b=0;
	switch(hi)
	{
		case 0: case 6: r=v; g=t; b=p; break;
		case 1: r=q; g=v; b=p; break;
		case 2: r=p; g=v; b=t; break;
		case 3: r=p; g=q; b=v; break;
		case 4: r=t; g=p; b=v; break;
		case 5: r=v; g=p; b=q; break;
	}
	
	rgb[0] = (uint8_t)(r*255);
	rgb[1] = (uint8_t)(g*255);
	rgb[2] = (uint8_t)(b*255);
}

static uint8_t sinTable[256] = {
	0x00,0x01,0x03,0x04,0x06,0x07,0x09,0x0a,0x0c,0x0e,0x0f,0x11,0x12,0x14,0x15,0x17,
	0x19,0x1a,0x1c,0x1d,0x1f,0x20,0x22,0x24,0x25,0x27,0x28,0x2a,0x2b,0x2d,0x2e,0x30,
	0x31,0x33,0x35,0x36,0x38,0x39,0x3b,0x3c,0x3e,0x3f,0x41,0x42,0x44,0x45,0x47,0x48,
	0x4a,0x4b,0x4d,0x4e,0x50,0x51,0x53,0x54,0x56,0x57,0x59,0x5a,0x5c,0x5d,0x5f,0x60,
	0x61,0x63,0x64,0x66,0x67,0x69,0x6a,0x6c,0x6d,0x6e,0x70,0x71,0x73,0x74,0x75,0x77,
	0x78,0x7a,0x7b,0x7c,0x7e,0x7f,0x80,0x82,0x83,0x84,0x86,0x87,0x88,0x8a,0x8b,0x8c,
	0x8e,0x8f,0x90,0x92,0x93,0x94,0x95,0x97,0x98,0x99,0x9a,0x9c,0x9d,0x9e,0x9f,0xa1,
	0xa2,0xa3,0xa4,0xa5,0xa7,0xa8,0xa9,0xaa,0xab,0xac,0xae,0xaf,0xb0,0xb1,0xb2,0xb3,
	0xb4,0xb5,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf,0xc0,0xc1,0xc2,0xc3,0xc4,
	0xc5,0xc6,0xc7,0xc8,0xc9,0xca,0xcb,0xcc,0xcd,0xce,0xcf,0xd0,0xd1,0xd1,0xd2,0xd3,
	0xd4,0xd5,0xd6,0xd7,0xd7,0xd8,0xd9,0xda,0xdb,0xdc,0xdc,0xdd,0xde,0xdf,0xdf,0xe0,
	0xe1,0xe2,0xe2,0xe3,0xe4,0xe4,0xe5,0xe6,0xe6,0xe7,0xe8,0xe8,0xe9,0xea,0xea,0xeb,
	0xec,0xec,0xed,0xed,0xee,0xee,0xef,0xef,0xf0,0xf1,0xf1,0xf2,0xf2,0xf3,0xf3,0xf3,
	0xf4,0xf4,0xf5,0xf5,0xf6,0xf6,0xf6,0xf7,0xf7,0xf8,0xf8,0xf8,0xf9,0xf9,0xf9,0xfa,
	0xfa,0xfa,0xfa,0xfb,0xfb,0xfb,0xfb,0xfc,0xfc,0xfc,0xfc,0xfd,0xfd,0xfd,0xfd,0xfd,
	0xfd,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xff
};

uint8_t sin4(uint8_t x)
{
	return sinTable[x];
}

void bscale(uint8_t* rgb, uint8_t scale)
{
	rgb[0] = ((uint16_t)rgb[0]*scale)/0xff;
	rgb[1] = ((uint16_t)rgb[1]*scale)/0xff;
	rgb[2] = ((uint16_t)rgb[2]*scale)/0xff;
}
